Package: BFpack
Type: Package
Title: Flexible Bayes Factor Testing of Scientific Expectations
Version: 1.6.0
Authors@R: 
    c(person(given = c("Joris"),
             family = "Mulder",
             role = c("aut", "cre"),
             email = "j.mulder3@tilburguniversity.edu"),
      person(given = "Caspar",
             family = "van Lissa",
             role = c("aut", "ctb"),
             email = "c.j.vanlissa@uu.nl"),
      person(given = "Donald R.",
             family = "Williams",
             role = c("aut", "ctb"),
             email = "drwwilliams@ucdavis.edu"),
      person(given = "Xin",
             family = "Gu",
             role = c("aut", "ctb"),
             email = "guxin57@hotmail.com"),
      person(given = "Anton",
             family = "Olsson-Collentine",
             role = c("aut", "ctb"),
             email = "J.A.E.OlssonCollentine@tilburguniversity.edu"),
      person(given = "Florian",
             family = "Boeing-Messing",
             role = c("aut", "ctb"),
             email = "F.Boeing-Messing@uvt.nl"),
      person(given = "Jean-Paul",
             family = "Fox",
             role = c("aut", "ctb"),
             email = "g.j.a.fox@utwente.nl"),
      person(given = "Janosch",
             family = "Menke",
             role = c("ctb"),
             email = "janosch.menke@uni-muenster.de"),
      person(given = "Robbie",
             family = "van Aert",
             role = c("ctb"),
             email = "R.C.M.vanAert@tilburguniversity.edu"),
      person(given = "Barry",
             family = "Brown",
             role = "ctb"),
      person(given = "James",
             family = "Lovato",
             role = "ctb"),
      person(given = "Kathy",
             family = "Russell",
             role = "ctb"),
      person("Lapack 3.8",
             role = "ctb"),
      person(given = "Jack",
             family = "Dongarra",
             role = "ctb"),
      person(given = "Jim",
             family = "Bunch",
             role = "ctb"),
      person(given = "Cleve",
             family = "Moler",
             role = "ctb"),
      person(given = "Gilbert",
             family = "Stewart",
             role = "ctb"),
      person(given = "John",
             family = "Burkandt",
             role = "ctb"),
      person(given = "Ashwith",
             family = "Rego",
             role = "ctb"),
      person(given = "Alexander",
             family = "Godunov",
             role = "ctb"),
      person(given = "Alan",
             family = "Miller",
             role = "ctb"),
      person(given = "Jean-Pierre",
             family = "Moreau",
             role = "ctb"),
      person("The R Core Team", role = "cph")
             )
Maintainer: Joris Mulder <j.mulder3@tilburguniversity.edu>
Description: Implementation of default Bayes factors
    for testing statistical hypotheses under various statistical models. The package is
    intended for applied quantitative researchers in the
    social and behavioral sciences, medical research,
    and related fields. The Bayes factor tests can be
    executed for statistical models such as 
    univariate and multivariate normal linear models,
    correlation analysis, generalized linear models, special cases of 
    linear mixed models, survival models, relational
    event models. Parameters that can be tested are
    location parameters (e.g., group means, regression coefficients),
    variances (e.g., group variances), and measures of 
    association (e.g,. polychoric/polyserial/biserial/tetrachoric/product
    moments correlations), among others.
    Relevant references on the methodology The statistical underpinnings are
    described in
    O'Hagan (1995) <DOI:10.1111/j.2517-6161.1995.tb02017.x>,
    Mulder and Xin (2022) <DOI:10.1080/00273171.2021.1904809>,
    Mulder and Gelissen (2019) <DOI:10.1080/02664763.2021.1992360>,
    Mulder and Fox (2019) <DOI:10.1214/18-BA1115>,
    Boeing-Messing, van Assen, Hofman, Hoijtink, and Mulder (2017) <DOI:10.1037/met0000116>,
    Hoijtink, Mulder, van Lissa, and Gu (2018) <DOI:10.1037/met0000201>,
    Gu, Mulder, and Hoijtink (2018) <DOI:10.1111/bmsp.12110>,
    Hoijtink, Gu, and Mulder (2018) <DOI:10.1111/bmsp.12145>, and
    Hoijtink, Gu, Mulder, and Rosseel (2018) <DOI:10.1037/met0000187>. When using the
    packages, please refer to the package Mulder et al. (2021) <DOI:10.18637/jss.v100.i18>
    and the relevant methodological papers.
License: GPL (>= 3)
URL: https://github.com/jomulder/BFpack
BugReports: https://github.com/jomulder/BFpack/issues
Encoding: UTF-8
LazyData: true
NeedsCompilation: yes
RoxygenNote: 7.3.3
Depends: R (>= 3.0.0), bain
Imports: stats, utils, MASS, mvtnorm, pracma, lme4, sandwich, QRM,
        coda, metaBMA, methods, berryFunctions, ergm, Bergm
Suggests: testthat, polycor, survival, pscl, metafor, knitr, rmarkdown,
        lmtest
VignetteBuilder: knitr
Packaged: 2026-04-13 20:54:28 UTC; jorismulder
Author: Joris Mulder [aut, cre],
  Caspar van Lissa [aut, ctb],
  Donald R. Williams [aut, ctb],
  Xin Gu [aut, ctb],
  Anton Olsson-Collentine [aut, ctb],
  Florian Boeing-Messing [aut, ctb],
  Jean-Paul Fox [aut, ctb],
  Janosch Menke [ctb],
  Robbie van Aert [ctb],
  Barry Brown [ctb],
  James Lovato [ctb],
  Kathy Russell [ctb],
  Lapack 3.8 [ctb],
  Jack Dongarra [ctb],
  Jim Bunch [ctb],
  Cleve Moler [ctb],
  Gilbert Stewart [ctb],
  John Burkandt [ctb],
  Ashwith Rego [ctb],
  Alexander Godunov [ctb],
  Alan Miller [ctb],
  Jean-Pierre Moreau [ctb],
  The R Core Team [cph]
Repository: CRAN
Date/Publication: 2026-04-13 21:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 13:34:24 UTC; unix
Archs: BFpack.so.dSYM
