# Generated by roxygen2: do not edit by hand

S3method(BF,bartlett_htest)
S3method(BF,bergm)
S3method(BF,coeftest)
S3method(BF,cor_test)
S3method(BF,coxph)
S3method(BF,default)
S3method(BF,ergm)
S3method(BF,glm)
S3method(BF,hetcor)
S3method(BF,htest)
S3method(BF,lm)
S3method(BF,lmerMod)
S3method(BF,mvt_test)
S3method(BF,polr)
S3method(BF,rma.uni)
S3method(BF,survreg)
S3method(BF,t_test)
S3method(BF,zeroinfl)
S3method(bartlett_test,default)
S3method(get_estimates,bartlett_htest)
S3method(get_estimates,bergm)
S3method(get_estimates,cor_test)
S3method(get_estimates,coxph)
S3method(get_estimates,ergm)
S3method(get_estimates,glm)
S3method(get_estimates,hetcor)
S3method(get_estimates,lm)
S3method(get_estimates,mvt_test)
S3method(get_estimates,polr)
S3method(get_estimates,survreg)
S3method(get_estimates,t_test)
S3method(get_estimates,zeroinfl)
S3method(plot,cor_test)
S3method(print,BF)
S3method(print,cor_test)
S3method(summary,BF)
S3method(summary,cor_test)
export(BF)
export(bartlett_test)
export(cor_test)
export(mvt_test)
import(bain)
importFrom(Bergm,bergm)
importFrom(MASS,fitdistr)
importFrom(MASS,ginv)
importFrom(QRM,fit.st)
importFrom(berryFunctions,is.error)
importFrom(coda,as.mcmc)
importFrom(coda,gelman.diag)
importFrom(coda,mcmc)
importFrom(coda,mcmc.list)
importFrom(ergm,ergmMPLE)
importFrom(lme4,VarCorr)
importFrom(lme4,getME)
importFrom(metaBMA,prior)
importFrom(methods,is)
importFrom(mvtnorm,dmvnorm)
importFrom(mvtnorm,dmvt)
importFrom(mvtnorm,pmvnorm)
importFrom(mvtnorm,pmvt)
importFrom(mvtnorm,rmvnorm)
importFrom(mvtnorm,rmvt)
importFrom(pracma,Rank)
importFrom(pracma,rref)
importFrom(sandwich,sandwich)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,bartlett.test)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cov)
importFrom(stats,dbeta)
importFrom(stats,density)
importFrom(stats,dlnorm)
importFrom(stats,dlogis)
importFrom(stats,dnorm)
importFrom(stats,dt)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qgamma)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rWishart)
importFrom(stats,rbeta)
importFrom(stats,rchisq)
importFrom(stats,rgamma)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,getFromNamespace)
importFrom(utils,globalVariables)
useDynLib(BFpack, .registration = TRUE)
