#' Student t approximations of Fisher transformed correlations
#'
#' Approximated degrees of freedom and approximated scale of the Fisher transformed
#' correlations depending on the dimension of the vector of dependent variables P
#' based on a joint uniform prior.
#'
#' \tabular{lll}{
#'    \strong{nu} \tab \code{numeric} \tab Approximated degrees of freedom\cr
#'    \strong{sigma} \tab \code{numeric} \tab Approximated scale\cr
#'    \strong{P} \tab \code{integer} \tab Dimension of vector of dependent variables\cr
#' }
#' @docType data
#' @keywords datasets
#' @name Fcor
#' @usage data(Fcor)
#' @format A data.frame with 3 columns.
NULL
