#' 
#' Data come from an experimental study (Rosa, Rosa, Sarner, and Barrett, 1998) 
#' that were also used in Howell (2012, p.196).
#' An experiment was conducted to investigate if Therapeutic Touch practitioners 
#' who were blindfolded can effectively identify which of their hands is below the experimenter¡¯s.
#' Twenty-eight practitioners were involved and tested 10 times in the experiment. 
#' Researchers expected an average of 5 correct answers from each practitioner 
#' as it is the number by chance if they do not outperform others.
#' 
#' \tabular{lll}{
#'    \strong{correct} \tab \code{integer} \tab How many correct answers are from each practitioner)\cr
#' }
#' @docType data
#' @keywords datasets
#' @name therapeutic
#' @usage data(therapeutic)
#' @references Howell, D. (2012). Statistical methods for psychology (8th ed.). Belmont, CA: Cengage Learning.
#' @format A data.frame with 22 rows and 1 column.
NULL
